#ifndef ALBUM_H
#define ALBUM_H

#include <iostream>
#include <set>
#include <algorithm>

using namespace std;

class EOFException {};

class Album
{
public:
  Album (istream &s)
  { s >> Id >> Title >> Artist >> Year;
    if (!s)  throw EOFException ();  // abort constructor!
    replace_if (Title.begin (), Title.end (), bind2nd (equal_to<char> (), '_'), ' ');
    replace_if (Artist.begin (), Artist.end (), bind2nd (equal_to<char> (), '_'), ' ');
  }

  Album (unsigned pId) { Id = pId; }  // for comparisons

  void Print (ostream &o = cout) const
  { o << Title << " (" << Artist << ", " << Year << ")\n"; }

  bool operator< (const Album &A) const  { return Id < A.Id; }

  bool PrintOrder (const Album *A) const
  { if (Artist < A->Artist) return true;
    if (Artist > A->Artist) return false;
    if (Year < A->Year) return true;
    if (Year > A->Year) return false;
    if (Title < A->Title) return true;
    if (Title > A->Title) return false;
    return false;
  }

private:
  unsigned Id;
  string Title, Artist;
  unsigned Year;
};

inline ostream &operator<< (ostream &o, const Album &A)
{ A.Print (o);  return o; }


class AlbumList
{
public:
  AlbumList (char *FileName);
  Album const *FindAlbum (unsigned Id) const
  { Album A (Id); return &*Albums.find (A); }
private:
  set<Album> Albums;
};

#endif
