#include <iostream>
#include <vector>
#include <algorithm>
#include "album.h"
#include <iomanip>

using namespace std;

class Track
{
public:
  Track (istream &s)
  { s >> Nr >> Title >> AlbumId;
    if (!s)  throw EOFException ();  // abort constructor!
    replace_if (Title.begin (), Title.end (), bind2nd (equal_to<char> (), '_'), ' ');
  }

  void Print (ostream &o = cout) const
  { o << "  " << setw (2) << setfill ('0') << Nr << ' ' << Title << endl; }

  void LookupAlbum (AlbumList &AL)
  { OnAlbum = AL.FindAlbum (AlbumId); }

  bool operator< (const Track &T) const
  { if (OnAlbum->PrintOrder (T.OnAlbum)) return true;
    if (T.OnAlbum->PrintOrder (OnAlbum)) return false;
    return Nr < T.Nr;
  }

  const Album *GetAlbum () const  { return OnAlbum; }

private:
  unsigned Nr;
  string Title;
  unsigned AlbumId;
  const Album *OnAlbum;
};

inline ostream &operator<< (ostream &o, const Track &T)
{ T.Print (o);  return o; }


class TrackList
{
public:
  TrackList (char *FileName, AlbumList &AL);
  void Print (ostream &o = cout) const;
private:
  vector<Track> Tracks;
  AlbumList &Albums;
};

inline ostream &operator<< (ostream &o, const TrackList &TL)
{ TL.Print (o);  return o; }
