#include <stdio.h>
#include <stdlib.h>
#include "track.h"
#include "string.h"

TrackList *NewTrackList (AlbumList *AL)
{
  TrackList *TL = (TrackList *) malloc (sizeof (TrackList));
  TL->Count = 0;
  TL->Allocated = 10;
  TL->Tracks = (Track **) calloc (TL->Allocated, sizeof (Track *));
  TL->Albums = AL;
  return TL;
}

void DeleteTrackList (TrackList *TL)
{
  int i;
  for (i = 0; i < TL->Count; i ++)  free (TL->Tracks[i]);
  free (TL->Tracks);
  free (TL);
}

Track *NewTrack (TrackList *TL)
{
  if (TL->Count >= TL->Allocated)
  { TL->Allocated *= 2;
    TL->Tracks = (Track **) realloc (TL->Tracks, TL->Allocated * sizeof (Track *));
  }
  Track *T = (Track *) malloc (sizeof (Track));
  TL->Tracks[TL->Count++] = T;
  return T;
}

void MakeAlbumPointers (TrackList *TL)
{
  int i;
  for (i = 0; i < TL->Count; i ++)
    TL->Tracks[i]->OnAlbum = FindAlbum (TL->Albums, TL->Tracks[i]->AlbumId);
}

int TrackOrder (const void *A, const void *B)
{
  int c;
  Track *TA = *(Track **) A;  Track *TB = *(Track **) B;
  if (A == B) return 0;
  c = strcmp (TA->OnAlbum->Artist, TB->OnAlbum->Artist);
  if (c != 0) return c;
  if (TA->OnAlbum->Year < TB->OnAlbum->Year) return -1;
  if (TA->OnAlbum->Year > TB->OnAlbum->Year) return +1;
  c = strcmp (TA->OnAlbum->Title, TB->OnAlbum->Title);
  if (c != 0) return c;
  if (TA->Nr < TB->Nr) return -1;
  return +1;
}

void SortTrackList (TrackList *TL)
{
  qsort (TL->Tracks, TL->Count, sizeof (Track *), &TrackOrder);
}

void ReadTrackList (TrackList *TL, char *FileName)
{
  int r, Nr;
  FILE *File = fopen (FileName, "r");
  if (File == NULL)  { perror (FileName);  exit (1); }
  do
  {
    r = fscanf (File, "%d", &Nr);
    if (r != EOF)
    {
      Track *T = NewTrack (TL);
      T->Nr = Nr;
      r = fscanf (File, "%100s %d", T->Title, &T->AlbumId);
      StringSubst (T->Title, '_', ' ');
    }
  }
  while (r != EOF);
  fclose (File);
  MakeAlbumPointers (TL);
  SortTrackList (TL);
}

void PrintTrackList (TrackList *TL)
{
  Album *A = NULL;
  int i;
  for (i = 0; i < TL->Count; i ++)
  {
    Track *T = TL->Tracks[i];
    if (T->OnAlbum != A)
    { A = T->OnAlbum;
      PrintAlbum (A);
    }
    PrintTrack (T);
  }
}

void PrintTrack (Track *T)
{
  printf ("  %02d %s\n", T->Nr, T->Title);
}
